/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebMail Notifier.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsIWebMailNotifier = Components.interfaces.nsIWebMailNotifier;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{e3ab0980-b4e1-11db-abbd-0800200c9a66}");

// description
const CLASS_NAME = "WebMail Notifier Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/WebMailNotifier;1";

const ST_PRE         = 0;
const ST_PRE_RES     = 1;
const ST_LOGIN       = 10;
const ST_LOGIN_RES   = 11;
const ST_DATA        = 20;
const ST_DATA_RES    = 21;

const LOGIN_URL      = 0;
const LOGIN_ID       = 1;
const LOGIN_PW       = 2;
const LOGIN_DATA     = 3;
const LOGIN_POST     = 1;

const dout=Components.utils.reportError;

/***********************************************************
class definition
***********************************************************/

//class constructor
function WebMailNotifier() {
  //preference observer
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  this.prefBranch = prefService.getBranch("wm-notifier.");
  this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.prefBranch.addObserver("", this, false);
  this.timerDelay=this.prefBranch.getIntPref("updateInterval")*60000;
  this.newMailsOnly=this.prefBranch.getBoolPref("newMailsOnly");
  this.initHandlers(false);

  var observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
  observerService.addObserver(this, "http-on-modify-request", false);
  observerService.addObserver(this, "http-on-examine-response", false);
}

// class definition
WebMailNotifier.prototype = {
  // define the function we want to expose in our interface
  listeners : new Array(),
  timer : null,
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIWebMailNotifier) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

WebMailNotifier.prototype.initHandlers = function(aCheckNow) {
  this.webMailHandlers=[];
  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                .getService(Components.interfaces.nsIPasswordManager);
  var e = passwordManager.enumerator;
  var str="chrome://wm-notifier/webmail";
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
  while (e.hasMoreElements()) {
    try {
      var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
      if (pass.host.indexOf(str)==0) {
        var id=pass.host.substring(str.length);
        var obj=new WebMailHandler(this);
        this.webMailHandlers.push(obj);
        obj.id=id;
        obj.user=pass.user;
        obj.password=pass.password;
        eval("initHandler"+id+"(this.webMailHandlers["+(this.webMailHandlers.length-1)+"])");
        obj.dataURI=ioService.newURI(obj.dataURL,null,null);
        var post=(obj.loginData[LOGIN_DATA]?obj.loginData[LOGIN_DATA]:"")
              +(obj.loginData[LOGIN_ID]?("&"+obj.loginData[LOGIN_ID]+"="+obj.user):"")
              +"&"+obj.loginData[LOGIN_PW]+"="+obj.password;
        obj.loginData.splice(1,obj.loginData.length-1);
        obj.loginData[LOGIN_POST]=post;
        obj.stage=obj.initStage;
      }
    }catch(ex) {
    }
  }
  this.webMailHandlers.sort(this.sortFunc);
  this.buildTable(aCheckNow);
}
WebMailNotifier.prototype.sortFunc = function(a,b) {
  if(a.id<b.id)return -1;
  else if(a.id==b.id){
    if(a.user<b.user)return -1;
    else if(a.user==b.user)return 0;
    else return 1;
  }else return 1;
}
WebMailNotifier.prototype.buildTable = function(aCheckNow) {
  this.hdlTable=[];
  var defaults=[];
  for(var i=0;i<nsIWebMailNotifier.NUM_OF_MAILS;i++){
    this.hdlTable.push([]);
    try{
        defaults[i]=this.prefBranch.getCharPref("webmail"+i+".default");
    }catch(e){}
  }

  for each(var o in this.webMailHandlers){
    try{
      o.enabled=this.prefBranch.getBoolPref("webmail"+o.id+"."+o.user+".enabled");
    }catch(e){}
    if(o.enabled){
      if(o.user==defaults[o.id])this.hdlTable[o.id].splice(0,0,o);//default account inserted into index 0
      else this.hdlTable[o.id].push(o);
    }
  }
  for(var i=0;i<this.hdlTable.length;i++){
    var o=this.hdlTable[i];
    if(o.length>=1){
      var prefStr="webmail"+i+".default";
      var t=null;
      try{
        t=this.prefBranch.getCharPref(prefStr);
      }catch(e){}
      if(t==null)this.prefBranch.setCharPref(prefStr,o[0].user);
    }
  }
  if(aCheckNow){
    this.setState(nsIWebMailNotifier.ST_RESET,"",0);
    this.checkMails();
  }
}
WebMailNotifier.prototype.isDefault = function(id,user) {
  return this.hdlTable[id].length==1||(this.hdlTable[id].length>1&&this.hdlTable[id][0].user==user);
}
WebMailNotifier.prototype.getAccountsNumber = function() {
  return this.webMailHandlers.length;
}
WebMailNotifier.prototype.getAccountInfo = function(aIndex) {
  var obj=this.webMailHandlers[aIndex];
  return obj.id+"\t"+obj.user+"\t"+obj.password;
}
WebMailNotifier.prototype.isAccountEnabled = function(aIndex) {
  return this.webMailHandlers[aIndex].enabled;
}
WebMailNotifier.prototype.addListener = function(aListener) {
  this.listeners.push(aListener);
  if (!this.timer){//first listener
    this.checkMails();
    this.setTimer();
  }else{
    for each(var i in this.webMailHandlers){
      var count=this.calcCount(i.id,i.user,i.mailCount);
      aListener.onStateChange(i.id,i.user,count);
    }
  }
}
WebMailNotifier.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
WebMailNotifier.prototype.checkMails = function() {
  for each(var o in this.webMailHandlers)o.checkMail();
}
WebMailNotifier.prototype.getHandler = function(aID,aUser) {
  for each(var i in this.webMailHandlers){
    if(i.id==aID&&i.user==aUser)return i;
  }
  return null;
}
WebMailNotifier.prototype.getMailURL = function(aID,aUser) {
  var obj=this.getHandler(aID,aUser);
  if(obj){
    obj.cookieManager.setCookieToBrowser();
    if(this.newMailsOnly){
      this.prefBranch.setIntPref("webmail"+aID+"."+aUser+".count",obj.mailCount);
      this.setState(aID,aUser,obj.mailCount);
    }
    return obj.mailURL;
  }else return null;
}
WebMailNotifier.prototype.calcCount = function(aID,aUser,aCount) {
  if(this.newMailsOnly){
    var prefStr="webmail"+aID+"."+aUser+".count";
    var count=null;
    try{
      count=this.prefBranch.getIntPref(prefStr);
    }catch(e){}
    if(count==null){
      this.prefBranch.setIntPref(prefStr,0);
      count=0;
    }
    if(aCount>=0){
      if(aCount>=count)aCount-=count;
      else{
        this.prefBranch.setIntPref(prefStr,aCount);
        aCount=0;
      }
    }
  }
  return aCount;
}
WebMailNotifier.prototype.setState = function(aID,aUser,aCount) {
  if(aID>=0)aCount=this.calcCount(aID,aUser,aCount);
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aID,aUser,aCount);
    }
  }
}

/***********************************************************
timer
***********************************************************/
WebMailNotifier.prototype.setTimer = function() {
  if (this.timer) {
    this.timer.cancel();
  } else {
    this.timer = Components.classes["@mozilla.org/timer;1"]
                  .createInstance(Components.interfaces.nsITimer);
  }
  if(this.timerDelay>0){
    try{
      this.timer.initWithCallback(this,
                                  this.timerDelay,
                                  Components.interfaces.TYPE_ONE_SHOT);
    }catch(e){}
  }
}

WebMailNotifier.prototype.notify = function(aTimer) {
  this.checkMails();
  this.setTimer();
}
/***********************************************************
load html
***********************************************************/
WebMailNotifier.prototype.getHtml = function(aHandler,aURL,aPostData) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(aURL, null, null);
  var channel = ioService.newChannelFromURI(uri);
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  if (aPostData) {
    var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                        .createInstance(Components.interfaces.nsIStringInputStream);
    uploadStream.setData(aPostData, aPostData.length);
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    httpChannel.requestMethod = "POST";
  }
  aHandler.channel=channel;
  channel.notificationCallbacks = aHandler;
  channel.asyncOpen(aHandler,httpChannel);
}
/***********************************************************
preference observer
***********************************************************/
WebMailNotifier.prototype.observe=function(aSubject, aTopic, aData)
{
  switch(aTopic){
  case "nsPref:changed":
    switch (aData) {
    case "updateInterval":
      this.timerDelay=this.prefBranch.getIntPref(aData)*60000;
      this.setTimer();
      break;
    case "newMailsOnly":
      this.newMailsOnly=this.prefBranch.getBoolPref("newMailsOnly");
      for each(var o in this.webMailHandlers){
        if(o.enabled){
          this.prefBranch.setIntPref("webmail"+o.id+"."+o.user+".count",0);
          this.setState(o.id,o.user,o.mailCount);
        }
      }
      break;
    }
    break;
  case "http-on-modify-request":
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.cookieRequest(aSubject.QueryInterface(Components.interfaces.nsIHttpChannel));
        break;
      }
    }
    break;
  case "http-on-examine-response":
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.cookieResponse(aSubject.QueryInterface(Components.interfaces.nsIHttpChannel));
        break;
      }
    }
    break;
  }
}
/***********************************************************
password manager
***********************************************************/
WebMailNotifier.prototype.getPassword=function(mailid,userid){
  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                .getService(Components.interfaces.nsIPasswordManager);
  var e = passwordManager.enumerator;
  var str="chrome://wm-notifier/webmail"+mailid;
  while (e.hasMoreElements()) {
    try {
      var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
      if (pass.host == str&&pass.user==userid) {
         return pass.password; // the password
      }
    }catch(ex) {
    }
  }
  return "";
}

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var WebMailNotifierFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new WebMailNotifier()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var WebMailNotifierModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return WebMailNotifierFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return WebMailNotifierModule; }

/***********************************************************
CookieManager definition
***********************************************************/
function CookieManager(){
  this.cookies=[];
}
CookieManager.prototype={
  addCookies: function(aURI,aCookie){
    aCookie=aCookie.split("\n");
    for each(var o in aCookie){
      var val={};
      var ar=o.split(";");
      var t=ar[0].match(/(\S+?)=(.*)/);

      val["name"]=t[1];
      val["value"]=t[2];

      for(var i=1;i<ar.length;i++){
        if(ar[i]){
          var t=ar[i].replace( /^\s+/g,"");
          var f=t.match(/(\S+?)=(.*)/);
          if(f)val[f[1].toLowerCase()]=f[2];
          else val[t.toLowerCase()]="";
        }
      }
      val.uri=aURI;
      this._insert(val);
    }
  },
  _insert: function(val){
    for(var i in this.cookies){
      var o=this.cookies[i];
      var domain=o.domain?o.domain:o.uri.host;
      var path=o.path?o.path:o.uri.path;
      var domain2=val.domain?val.domain:val.uri.host;
      var path2=val.path?val.path:val.uri.path;
      if(o.name==val.name&&domain==domain2&&path==path2){
        this.cookies[i]=val;
        return;
      }
    }
    this.cookies.push(val);
  },
  findCookie:function(urispec,name){
    for each(var o in this.cookies){
      if(o.uri.spec==urispec&&o.name==name){
        return this.toString(o);
      }
    }
    return null;
  },
  getCookie: function(aURI){
    var str="";
    for each(var o in this.cookies){
      var domain=o.domain;
      if(!domain)domain=o.uri.host;
      if(aURI.host.lastIndexOf(domain)==-1)continue;
      var path=o.path;
      if(!path)path=o.uri.path;
      if(aURI.path.indexOf(path)!=0)continue;
      var exp=o.expires;
      if(exp){
        exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
        if(Date.parse(exp)<new Date()){
          continue;
        }
      }
      var s=o.name+"="+o.value;
      if(str)str+="; "+s;
      else str=s;
    }
    return str;
  },
  toString:function(o){
      var s=o.name+"="+o.value;
      for(var p in o){
        if(p!="name"&&p!="value"&&p!="uri"&&p!="httponly"&&p!="secure"){
          s+="; "+p+"="+o[p];
        }
      }
    return s;
  },
  setCookieToBrowser: function(){
    var cs = Components.classes["@mozilla.org/cookieService;1"]
                  .getService(Components.interfaces.nsICookieService);
    for each(var o in this.cookies){
      cs.setCookieString(o.uri,null,this.toString(o), null);
    }
  },
  clear: function(){
    this.cookies=[];
  }
}
/***********************************************************
WebMailHandler definition
***********************************************************/
function WebMailHandler(webMailNotifier){
  this.webMailNotifier=webMailNotifier;
  this.cookieManager=new CookieManager();
  this.loggedIn=false;
}
WebMailHandler.prototype={
  charSet: null,
  data: "",
  stage: 0,
  initStage: ST_LOGIN,
  mailCount:-1,
  user: null,
  password: null,

  // nsIChannelEventSink
  onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
    if(this.channel==aOldChannel)this.channel = aNewChannel;
  },
  // nsIInterfaceRequestor
  getInterface: function (aIID) {
    try {
      return this.QueryInterface(aIID);
    } catch (e) {
      throw Components.results.NS_NOINTERFACE;
    }
  },
  // nsIProgressEventSink
  onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },
  onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },
  // nsIHttpEventSink
  onRedirect : function (aOldChannel, aNewChannel) { },
  QueryInterface : function(aIID) {
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||
        aIID.equals(Components.interfaces.nsIChannelEventSink) ||
        aIID.equals(Components.interfaces.nsIProgressEventSink) ||
        aIID.equals(Components.interfaces.nsIHttpEventSink) ||
        aIID.equals(Components.interfaces.nsIStreamListener)||
        aIID.equals(Components.interfaces.nsIPrompt)|| //not used but required
        aIID.equals(Components.interfaces.nsIAuthPrompt)||
        aIID.equals(Components.interfaces.nsIAuthPromptProvider))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },
  onStartRequest : function (aRequest, aContext) {
    this.data = "";
  },
  onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
	  var is = Components.classes["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Components.interfaces.nsIScriptableInputStream);
  	is.init(aStream);
  	this.data += is.read(aLength);
  },
  onStopRequest : function (aRequest, aContext, aStatus) {
    var charSet=null;
    var httpChannel=this.channel.QueryInterface(Components.interfaces.nsIHttpChannel);
    if(this.charSet)charSet=this.charSet;
    else{
      var fnd=this.data.match(/<meta.+?\".+?charset=(\S+?)\s*?\"\s*?\x2f*>/);
      if(fnd)charSet=fnd[1];
      else{
        try{
          if(httpChannel.contentCharset)charSet=httpChannel.contentCharset;
        }catch(e){}
      }
    }
    if(charSet){
      var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
      								.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      uniConv.charset = charSet;
      try{
        this.data=uniConv.ConvertToUnicode(this.data);
      }catch(e){}
    }
    if(!this.process(aRequest, aContext, aStatus))++this.stage;
  },
  process : function (aRequest, aContext, aStatus){
    switch(this.stage){
    case ST_LOGIN:
      this.webMailNotifier.getHtml(this,this.loginData[LOGIN_URL],this.loginData[LOGIN_POST]);
      break;
    case ST_LOGIN_RES:
      this.stage=ST_DATA;
    case ST_DATA:
      this.webMailNotifier.getHtml(this,this.dataURL);
      break;
    case ST_DATA_RES:
      this.mailCount=this.getData(this.data);
      this.stage=this.mailCount<0?this.initStage:ST_DATA;
      this.webMailNotifier.setState(this.id,this.user,this.mailCount);
      if(this.mailCount>=0&&!this.loggedIn&&this.webMailNotifier.isDefault(this.id,this.user)){
        this.loggedIn=true;
        this.cookieManager.setCookieToBrowser();
      }
      return true;
    }
  },
  checkMail : function(){
    if(!this.enabled)return;
    if(!this.process(null,null,0))++this.stage;
  },
  cookieRequest : function(aHttpChannel){
    if(this.stage==this.initStage){
      aHttpChannel.setRequestHeader("Cookie", "", false);//try clean login
      this.cookieManager.clear();
    }else{
      var ck=this.cookieManager.getCookie(aHttpChannel.URI);
      aHttpChannel.setRequestHeader("Cookie",ck, false);
    }
  },
  cookieResponse : function(aHttpChannel){
    try {
      var cookie = aHttpChannel.getResponseHeader("Set-Cookie");
      this.cookieManager.addCookies(aHttpChannel.URI,cookie);
      aHttpChannel.setResponseHeader("Set-Cookie", "", false);
    }catch(e){}
  }
};
/***********************************************************
daum(hanmail)
***********************************************************/
function initHandler0(handler){
  handler.dataURL="http://mail.daum.net/hanmail/Welcome.daum?cid=&msg=";
  handler.loginData=["https://logins.daum.net/Mail-bin/login.cgi",
                      "id","pw","url="+encodeURIComponent(handler.dataURL)];
  handler.mailURL="http://mail.daum.net/hanmail/Index.daum";

  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc0c8\ud3b8\uc9c0[\s\S]+?>(\d+)<\/a>\ud1b5/);
    if(fnd){
      return fnd[1];
    }else{
      return -1;
    }
  };
}
/***********************************************************
naver
***********************************************************/
function initHandler1(handler){
  handler.dataURL="http://mail.naver.com";
  handler.loginData=["https://nid.naver.com/nidlogin.login",
                      "id","pw",
                      "url="+encodeURIComponent(handler.dataURL)+"&cert=no&nomap=0&saveID=0&ssllogin=1"];
  handler.mailURL="http://mail.naver.com/list/?folder=0&m=1";

  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc548\uc77d\uc740\u0020\uba54\uc77c.+?<span>(\d+)<\/span>/);
    if(fnd){
      return fnd[1];
    }else{
      return -1;
    }
  };
}
/***********************************************************
empas
***********************************************************/
function initHandler2(handler){
  handler.dataURL="http://mail.empas.com/kr/main.html";
  handler.loginData=["https://login.empas.com/session/dologin.html",
                      "userid","passwd",
                      "es="+encodeURIComponent(handler.dataURL)];
  handler.mailURL="http://mail.empas.com/kr/mbox.html?n=1";

  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc548\u0020\uc77d\uc740\u0020\ud3b8\uc9c0[\s\S]*?(\d+)\ud1b5/);
    if(fnd){
      return fnd[1];
    }else{
      return -1;
    }
  };
}

/***********************************************************
nate
***********************************************************/
function initHandler3(handler){
  handler.dataURL="http://mail.nate.com/mail_frame_left.html";
  var ar=handler.user.split("@");
  handler.loginData=["https://xo.nate.com/servlets/LoginServlet",
                      "","PASSWD",
                      "redirect="+encodeURIComponent(handler.dataURL)
                      +"&ID="+encodeURIComponent(ar[0])+"&domain="+encodeURIComponent(ar[1])];
  handler.mailURL="http://mail.nate.com/";

  handler.getData=function(aHtml){
    var fnd=aHtml.match(/\uc804\uccb4\ubc1b\uc740\uba54\uc77c([\s\S]+?)<\/td>/);
    if(fnd){
      fnd=fnd[1].match(/\([\s\S]+?(\d+)[\s\S]+?\)/);
      if(fnd){
        return fnd[1];
      }else return 0;
    }
    return -1;
  };
}
/***********************************************************
Gmail
***********************************************************/
function initHandler4(handler){
  handler.dataURL="https://mail.google.com/mail/feed/atom";;
  handler.loginData=["https://www.google.com/accounts/ServiceLoginAuth",
                      "Email","Passwd",
                      "ltmpl=ca_tlsosm_t&ltmplcache=2&continue=http%3A%2F%2Fmail.google.com%2Fmail%2F%3F"
                        +"&service=mail&rm=false&rmShown=1&null=Sign+in"];
  handler.mailURL="http://mail.google.com/mail";

  handler.process=function (aRequest, aContext, aStatus) {
    switch(this.stage){
    case ST_LOGIN_RES:
      this.webMailNotifier.getHtml(this,"http://mail.google.com/mail");
      return false;
    case (ST_LOGIN_RES+1):
      this.stage=ST_DATA;
      break;
    }
    return WebMailHandler.prototype.process.call(this,aRequest, aContext, aStatus);
  }
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<fullcount>(\d+)<\/fullcount>/);
    if(fnd){
      return fnd[1];
    }else{
      this.mailchecked=false;
      return -1;
    }
  };
}
/***********************************************************
Yahoo
***********************************************************/
function initHandler5(handler){
  handler.initStage=ST_PRE;
  handler.dataURL="http://mail.yahoo.com/"
  handler.loginData=["https://login.yahoo.com/config/login?",
                      "login","passwd",""];
  handler.mailURL="http://mail.yahoo.com/"

  handler.process=function (aRequest, aContext, aStatus) {
    switch(this.stage){
    case ST_PRE:
      this.webMailNotifier.getHtml(this,"http://mail.yahoo.com");
      return false;
    case ST_PRE_RES:
      this.stage=ST_LOGIN;
      break;
    case ST_LOGIN_RES:
      var fnd=this.data.match(/<input.+?type="hidden".+?name=".secdata".+?value="(.+?)".*?>/);
      if(fnd){
        if(this.webMailNotifier.listeners.length>0){
          var rs=this.webMailNotifier.listeners[0].openYahooDialog(this.user,fnd[1]);
          if(rs){
            this.webMailNotifier.getHtml(this,this.loginData[LOGIN_URL],this.loginData[LOGIN_POST]+"&.secdata="+fnd[1]+"&.secword="+rs);
            return false;
          }
        }
        this.stage=this.initStage;
        this.webMailNotifier.setState(this.id,this.user,nsIWebMailNotifier.ST_RESET);
        return true;
      }
      this.stage=ST_DATA;
      break;
    case (ST_LOGIN_RES+1):
      this.stage=ST_DATA;
      break;
    }
    return WebMailHandler.prototype.process.call(this,aRequest, aContext, aStatus);
  };
  handler.cookieRequest=function(aHttpChannel){
    WebMailHandler.prototype.cookieRequest.call(this,aHttpChannel);
    if(this.stage==this.initStage){
      try{
        var s=this.webMailNotifier.prefBranch.getCharPref("webmail"+this.id+"."+this.user+".cookie");
        s=s.split("\t");
        var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
        var uri=ioService.newURI(s[0],null,null);
        this.cookieManager.addCookies(uri,s[1]);
      }catch(e){}
    }
  };
  handler.cookieResponse=function(aHttpChannel){
    WebMailHandler.prototype.cookieResponse.call(this,aHttpChannel);
    if(this.stage==(ST_LOGIN_RES+1)){
      var s=this.cookieManager.findCookie(aHttpChannel.URI.spec,"F");
      if(s){
        this.webMailNotifier.prefBranch.setCharPref("webmail"+this.id+"."+this.user+".cookie",aHttpChannel.URI.spec+"\t"+s);
      }
    }
  };
  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<div.+?folderlist.+?>([\s\S]+?)<\/div>\s*?<\/div>/);
    if(fnd){//original mode
      fnd=fnd[1].match(/<li .+?<\/a>/g);
      if(fnd){
        var s;
        var num=0;
        for each(s in fnd){
          var t=s.match(/id=\"(.+?)\"/);
          if(t){
            if(t[1]=="draft"||t[1]=="sent"
              ||t[1]=="bulk"||t[1]=="trash")continue;
          }
          s=s.match(/<\/div>.+?\((\d+)\)<\/a>/);
          if(s)num+=parseInt(s[1]);
        }
        return num;
      }
    }else{
      fnd=aHtml.match(/<ListFoldersResponse>([\s\S]+?)<\/ListFoldersResponse>/);
      if(fnd){
        fnd=fnd[1].match(/<folder.+?<\/folder>/g);
        if(fnd){
          var s;
          var num=0;
          for each(s in fnd){
            s=s.match(/unread=\"(\d+)\".+?fid=\"(.+?)\"/);
            if(s){
              if(s[2]=="%40B%40Bulk"||s[2]=="Draft"
                ||s[2]=="Sent"||s[2]=="Trash")continue;
              num+=parseInt(s[1]);
            }
          }
          return num;
        }
      }
    }
    return -1;
  };
}
/***********************************************************
Live(Hotmail)
***********************************************************/
function initHandler6(handler){
  handler.initStage=ST_PRE;
  handler.dataURL="http://mail.live.com/mail/InboxLight.aspx";
  handler.loginData=["",
                      "login","passwd",""];
  handler.mailURL="http://mail.live.com/";

  handler.process=function (aRequest, aContext, aStatus) {
    switch(this.stage){
    case ST_PRE:
      this.webMailNotifier.getHtml(this,"http://login.live.com/logout.srf");
      return false;
    case ST_PRE_RES:
      this.webMailNotifier.getHtml(this,"http://mail.live.com");
      return false;
    case (ST_PRE_RES+1):
      var fnd=this.data.match(/srf_uPost=\'([\s\S]+?)\'/);
      if(fnd){
        this.loginData[LOGIN_URL]=fnd[1];
        fnd=this.data.match(/PPFT[\s\S]+?value=\"(\S+?)\"/);
        if(fnd){
          this.stage=ST_LOGIN;
          this.webMailNotifier.getHtml(this,this.loginData[LOGIN_URL],this.loginData[LOGIN_POST]+"&PPFT="+fnd[1]);
          return false;
        }
      }
      this.stage=this.initStage;
      this.webMailNotifier.setState(this.id,this.user,nsIWebMailNotifier.ST_RESET);
      return true;
    }
    return WebMailHandler.prototype.process.call(this,aRequest, aContext, aStatus);
  }

  handler.getData=function(aHtml){
    var fnd=aHtml.match(/<ul.+?folderList.+?>([\s\S]+?)<\/ul>/);
    if(fnd){
      fnd=fnd[1].match(/InboxLight.aspx\?FolderID=.+?<\/a>/g);
      if(fnd){
        var s;
        var num=0;
        for each(s in fnd){
          s=s.match(/FolderID=.+?([^-]+?)&.+?\/>(.+?)<\/a>/);
          if(s){
            if(s[1]=="000000000005"||s[1]=="000000000004"
              ||s[1]=="000000000003"||s[1]=="000000000002")continue;
            s=s[2].match(/\((\d+)\)/);
            if(s)num+=parseInt(s[1]);
          }
        }
        return num;
      }
    }
    this.key=null;
    return -1;
  };
}


